<?php

class ControllerExtensionPaymentBetatransfer extends Controller
{
    private $order;
    private static $baseUrl = 'https://merchant.betatransfer.io';

    public function __construct($registry)
    {
        parent::__construct($registry);
        $this->load->language('extension/payment/betatransfer');
    }

    public function index()
    {
        $data = array();
        $data['button_confirm'] = $this->language->get('button_confirm');
        $data = array_merge($data, $this->makeForm());

        return $this->load->view('extension/payment/betatransfer', $data);
    }

    public function callback()
    {
        if ($this->request->server['REQUEST_METHOD'] == 'POST') {
            $bt_response = $this->request->post;
            $order_id = (int) $bt_response['orderId'];

            $this->load->model('checkout/order');
            $order = $this->model_checkout_order->getOrder($order_id);

            if (!$order) {
                $this->sendForbidden(sprintf($this->language->get('text_error_order_not_found'), $order_id));
                return false;
            }

            $bt_sign = $this->generateCallbackSign($bt_response);

            if ($bt_response['sign'] == $bt_sign) {
                if (in_array($bt_response['status'], array('success', 'partial_payment'))) {
                    $this->model_checkout_order->addOrderHistory(
                        $order_id,
                        $this->config->get('betatransfer_order_status_success'),
                        sprintf($this->language->get('text_comment'), $this->request->post['paymentSystem'], $this->request->post['amount']),
                        true
                    );
                } else {
                    $this->model_checkout_order->addOrderHistory(
                        $order_id,
                        $this->config->get('betatransfer_order_status_fail'),
                        sprintf($this->language->get('text_comment'), $this->request->post['paymentSystem'], $this->request->post['amount']),
                        true
                    );
                }

                echo 'OK';
                header("HTTP/1.1 200 OK");
            } else {
                $this->sendForbidden($this->language->get('text_error_bt_sign_hash'));
            }
        }
    }

    public function success()
    {
        if (!isset($this->session->data['order_id'])) {
            $this->session->data['order_id'] = $this->order['order_id'];
        }

        $this->response->redirect($this->url->link('checkout/success', '', 'SSL'));
    }

    public function fail()
    {
        if (!empty($this->session->data['order_id']) && $this->config->get('betatransfer_order_status_confirm') && ($this->session->data['payment_method']['code'] == 'betatransfer')) {
            $this->load->model('checkout/order');
            $this->model_checkout_order->addOrderHistory($this->session->data['order_id'], $this->config->get('betatransfer_order_status_fail'));
            $this->response->redirect($this->url->link('checkout/failure', '', 'SSL'));
        }
    }

    public function confirm()
    {
        if ($this->request->post['flag'] == 1) {
            if (!empty($this->session->data['order_id']) && $this->config->get('betatransfer_order_status_confirm') && ($this->session->data['payment_method']['code'] == 'betatransfer')) {
                $this->load->model('checkout/order');
                $this->model_checkout_order->addOrderHistory($this->session->data['order_id'], $this->config->get('betatransfer_order_status_confirm'), '', true);
                $json = '';
            }
        } else {
            $json = 'error';
        }

        $this->response->setOutput(json_encode($json));
    }

    protected function makeForm()
    {
        $this->load->model('checkout/order');

        if (isset($this->session->data['order_id'])) {
            $order_id = $this->session->data['order_id'];
        } else {
            die($this->language->get('error_fail_checkout_extension'));
        }

        $order_info = $this->model_checkout_order->getOrder($order_id);

        $bt_currency = $this->config->get('betatransfer_currency');

        if (!$this->currency->has($bt_currency)) {
            die(sprintf($this->language->get('text_error_currency_not_found'), $bt_currency));
        }

        if ($this->config->get('betatransfer_payment_system')) {
            $payment_systems = $this->sendRequest('/api/pay-systems', [
                'currency' => $bt_currency,
                'plugin' => 'opencart',
            ]);

            if (empty($systems) || !in_array($this->config->get('betatransfer_payment_system'), array_column($payment_systems, 'name'))) {
                die($this->language->get('text_error_payment_system_not_found'));
            }
        }

        $form_data = array_filter(array(
            'amount' => number_format($this->currency->convert($order_info['total'], $this->config->get('config_currency'), $bt_currency), 2, '.', ''),
            'currency' => $bt_currency,
            'orderId' => $order_id,
            'locale' => $this->language->get('code'),
            'urlSuccess' => HTTPS_SERVER . 'index.php?route=extension/payment/betatransfer/success',
            'urlFail' => HTTPS_SERVER . 'index.php?route=extension/payment/betatransfer/fail',
            'urlResult' => HTTPS_SERVER . 'index.php?route=extension/payment/betatransfer/callback',
            'payerId' => hash('crc32', $order_info['customer_id']),
            'payerPhone' => !empty($order_info['telephone']) ? $order_info['telephone'] : null,
            'payerEmail' => !empty($order_info['email']) ? $order_info['email'] : null,
            'payer_firstname' => !empty($order_info['firstname']) ? $order_info['firstname'] : null,
            'payer_lastname' => !empty($order_info['lastname']) ? $order_info['lastname'] : null,
            'fullCallback' => 1,
        ));

        if ($this->config->get('betatransfer_payment_system')) {
            $form_data['paymentSystem'] = $this->config->get('betatransfer_payment_system');
        }

        $form_data['sign'] = $this->generateSign($form_data);
        $response = $this->sendRequest('/api/payment', $form_data);

        if (empty($response['urlPayment'])) {
            die($this->language->get('text_error_gateway'));
        }

        $url = parse_url($response['urlPayment']);
        $action = $url['scheme'] . '://' . $url['host'];

        if (!empty($url['path'])) {
            $action = $action . $url['path'];
        }

        $parameters = array();

        if (!empty($url['query'])) {
            parse_str($url['query'], $parameters);
        }

        $data = array();
        $data['method'] = 'GET';
        $data['action'] = $action;
        $data['parameters'] = $parameters;

        return $data;
    }

    protected function sendForbidden($error)
    {
        $this->response->addHeader('HTTP/1.1 403 Forbidden');
        echo "<html>
                <head>
                   <title>403 Forbidden</title>
                </head>
                <body>
                    <p>$error.</p>
                </body>
        </html>";
    }

    protected function sendRequest($action, $params)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, self::$baseUrl . $action . '?token=' . $this->config->get('betatransfer_token'));
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        $response = curl_exec($ch);
        curl_close($ch);

        return json_decode($response, true);
    }

    private function generateSign($data)
    {
        if (!empty($data['sign'])) {
            unset($data['sign']);
        }

        array_push($data, $this->config->get('betatransfer_secret'));
        return md5(implode('', $data));
    }

    private function generateCallbackSign($data)
    {
        $data = array_filter($data, function($key) {
            return in_array($key, array('amount', 'orderId'));
        }, ARRAY_FILTER_USE_KEY);

        ksort($data);

        return $this->generateSign($data);
    }
}

?>
